#include <math.h>
#define dabs(x) (x<0 ? -x : x)
double cdtr(double chisq, int idf, int *ier) {
/* C  *DECK CDTR
C
C     ..................................................................
C
C        SUBROUTINE CDTR
C
C        PURPOSE
C           COMPUTES P(CHISQ) = PROBABILITY THAT THE RANDOM VARIABLE U,
C           DISTRIBUTED ACCORDING TO THE CHI-SQUARE DISTRIBUTION WITH G
C           DEGREES OF FREEDOM, IS LESS THAN OR EQUAL TO CHISQ.  F(G,CHI
C           ORDINATE OF THE CHI-SQUARE DENSITY AT CHISQ, IS ALSO COMPUTE
C
C        USAGE
C           CALL CDTR(CHISQ,IDF,SIGCHI,IER)
C
C        DESCRIPTION OF PARAMETERS
C           CHISQ   - INPUT SCALAR FOR WHICH P(CHISQ) IS COMPUTED.
C           IDF     - NUMBER OF DEGREES OF FREEDOM OF THE CHI-SQUARE
C                     DISTRIBUTION.  IDF IS AN INTEGER.
C           SIGCHI  - OUTPUT PROBABILITY.
C           IER     - RESULTANT ERROR CODE WHERE
C               IER= 0 --- NO ERROR
C               IER=-1 --- AN INPUT PARAMETER IS INVALID.  X IS LESS
C                          THAN 0.0, OR IDF IS LESS THAN 0.0 OR GREATER
C                          THAN 2*10**(+5).  P AND D ARE SET TO -1.E25.
C               IER=-2 --- IDF = 0, P SET TO 0.
C               IER=+1 --- INVALID OUTPUT.  P IS LESS THAN ZERO OR
C                          GREATER THAN ONE, OR SERIES FOR T1 (SEE
C                          MATHEMATICAL DESCRIPTION) HAS FAILED TO
C                          CONVERGE.  P IS SET TO 1.E25.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NDTR
C
C        METHOD
C
C     THIS FUNCTION WAS LIFTED FROM "NWAY" WRITTEN BY THE STANFORD
C     COMPUTATION CENTER. THE SCC BORROWED IT FROM JOHN MORRIS OF THE
C     COMPUTER INSTITUTE FOR SOCIAL SCIENCE RESEARCH AT MICHIGAN STATE
C     UNIVERSITY.
   C */
    int i,crap;
    double sigchi=0,anschi=1,ff,p,xi,pp,term,argnrm,gam=1,txx;
	double ndtr(double x);
    //   FACTORIALS OF 0,0 THROUGH 30. .
    double fact[]={1.,1.,2.,6.,24.,120.,720.,5040.,40320.,
        362880.,3628800.,3.99168e7,4.790016e8,6.2270208e9,
        8.71782912e10,1.307674368e12,2.0922789888e13,3.55687428096e14,
        6.402373705728e15,1.21645100408832e17,2.43290200817664e18,
        5.10909421717094e19,1.12400072777761e21,2.58520167388850e22,
        6.20448401733239e23,1.55112100433310e25,4.03291461126606e26,
        1.08888694504184e28,3.04888344611714e29,8.84176199373970e30,
        2.65252859812191e32};
	*ier=0;
    if (chisq<0 || idf<0) { *ier=-1;  sigchi=-1.e25;  }
    if (idf>0 && (chisq+100.)!=100.) {
        //      GT 60 D.F. OR CHI GT 100 - USE A NORMAL APPROXIMATION
        if (idf>60. || chisq>100.) {
            argnrm=sqrt(2.*chisq)-sqrt(2.*(double) idf -1.);
            sigchi=ndtr(argnrm);
        }
        else {
            ff=idf-2; p=0.5*ff; xi=0.5*chisq; pp=p+2; term=xi/pp;
            for (i=1; i<=100; i++) {
				anschi+=term;
				if (dabs(term)<.000001) break;
				term*=(xi/++pp);
			}
            gam=1; crap=idf&1;
            if ((idf&1)==0)  // if even
				gam=fact[idf/2];
            else {
                for (i=1; i<=idf; i+=2) gam*=((double) i)*0.5;
                gam*=1.7724588509;
            }
            txx=(p+1.)*log(xi)- xi + log(anschi/gam);
            if (txx<=88.)
                if (txx>-88.) anschi=exp(txx);
                else anschi=0.;
            else anschi=1.;
            sigchi=anschi;
        }
        if (sigchi<0 || sigchi>1.) { *ier=1; sigchi=1.e25; }
    }
    return(sigchi);
}
