#include <stdlib.h>
#include <stdio.h>
double cspool(double *observ, double *expect, int *ndf, int ncell) {
/*    C     SUBROUTINE CSPOOL PERFORMS POOLING OF CHI-SQUARE TABLES
      SUBROUTINE CSPOOL(OBSERV,EXPECT,NDF,NCELL,CHISQ)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION OBSERV(1), EXPECT(1), CHISQ
      INTEGER NDF, NCELL, MP, ML, MU, I, IPL
C   OBSERV  - ARRAY OF OBSERVED CELL FREQUENCIES
C   EXPECT  - ARRAY OF EXPECTED CELL FREQUENCIES
C   NDF    - NUMBER OF DEGREES OF FREEDOM IN TABLE
C            (UNITY SUBSTRACTED EACH TIME TWO CELLS ARE POOLED)
C   NCELL  - NUMBER OF CELLS IN TABLE, OR LENGTH OF OBSERV AND EXPECT
C   CHISQ  - CALCULATED CHI-SQUARE STATISTIC AFTER POOLING
C*/
    double chisq=0;    int i,mu,ml,ipl,mp,flg=0,ndfx;
    mu=ncell; ndfx=*ndf;
    // C   FIND THE MINIMUM EXPECTED VALUE, AND POOL WITH SMALLEST NEIGHBOR
    while (flg==0) {
        for (ml=1,i=2; i<=mu; i++) if (expect[ml]>expect[i]) ml=i;
        if (mu>2 && expect[ml-1]<1.9999 && ndfx>1) {
            if (ml==1) mp=2;
            else 
				if (ml==mu) mp=mu-1;
				else 
					if (expect[ml-1]>expect[ml+1]) mp=ml+1;
					else mp=ml-1;
            ipl=((mp<ml)? mp : ml);
            expect[mp]=expect[mp]+expect[ml];
            observ[mp]=observ[mp]+observ[ml];
            for (i=ml; i<mu; i++) {expect[i]=expect[i+1]; observ[i]=observ[i+1];}
            mu--; ndfx--;
        }
        else {
            //       CALCULATE CHI-SQUARE STATISTIC
            for (i=1; i<=mu; i++) {
				if (expect[i]>0.) chisq+=(observ[i]-expect[i])*(observ[i]-expect[i])/expect[i];
			}
			flg=1;
		}
    }
    *ndf=ndfx; return(chisq);
}
