/*  genmsstl.c - Generates capture history records for MSSURVIV w/ tag loss */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifdef __BC__
#include <dir.h> 
#include <alloc.h>
#endif
#include <math.h>
#include <process.h>
#include <string.h>
FILE *fopen(), *fp;
#define MXPER 12
#define MXAREAS 4
#define EPS .001

int nper, nper1, fper, nareas, determ, debug=0, nages=1, *iage;
double ***pa,***sa,****mv,****lam,***na;
char cage[]="jaxxxxxxxxxxxxx";

void move(double n, int per, int area, int age, int fage, char *caphis);
void cap(double n, int per, int area, int age, int fage, char *caphis);

void srv(double n,int per, int area, int age, int fage, char *caphis)
{ int i,iarea; double ndead, nsrv, nreported, treported=0;
    if (debug) printf("srv:n=%f per=%d area=%d age=%d fage=%d s=%f caphis=%s\n",n,per,area,age,fage,sa[age][per][area],caphis);
    ndead=n*(1.-sa[age][per][area]);
    for (iarea=0; iarea<nareas; iarea++) {
        nreported=ndead*lam[age][per][area][iarea]; treported+=nreported;
        if (nreported>EPS && strpbrk(caphis,"ABCDEFGHIJKLMNOPQRSTUVWXYZ")!=NULL) {
            for (i=0; i<fper; i++) fprintf(fp,"0"); fprintf(fp,"%s",caphis);
            for (i=per+1; i<nper; i++) fprintf(fp,"0"); fprintf(fp," %c %2d %c %12.6f\n",cage[fage],per+1,'A'+iarea,nreported);
            if (debug) {
                for (i=0; i<fper; i++) printf("0"); printf("%s",caphis);
                for (i=per+1; i<nper; i++) printf("0");  printf(" %c %2d %c %12.6f\n",cage[fage],per+1,'A'+iarea,nreported);
            }
        }
    }
    ndead=ndead-treported;
    if (ndead>EPS && strpbrk(caphis,"ABCDEFGHIJKLMNOPQRSTUVWXYZ")!=NULL) {
        for (i=0; i<fper; i++) fprintf(fp,"0"); fprintf(fp,"%s",caphis);
        for (i=per+1; i<nper; i++) fprintf(fp,"0"); fprintf(fp," %c %2d %c %12.6f\n",cage[fage],0,' ',ndead);
        if (debug) {
            for (i=0; i<fper; i++) printf("0"); printf("%s",caphis);
            for (i=per+1; i<nper; i++) printf("0");  printf(" %c %2d %c %12.6f\n",cage[fage],0,' ',ndead);
        }
    }
    nsrv=n-ndead-treported;
    if (per<(nper-1)) {
        if (nsrv>EPS && nsrv>0) move(nsrv,per,area,age,fage,caphis);
    }
    else {
        if (nsrv>EPS && strpbrk(caphis,"ABCDEFGHIJKLMNOPQRSTUVWXYZ")!=NULL) {
            for (i=0; i<fper; i++) fprintf(fp,"0"); fprintf(fp,"%s %c %2d %c %12.6f\n",caphis,cage[fage],0,' ',nsrv);
            if (debug) { for (i=0; i<fper; i++) printf("0");  printf("%s %c %2d %c %12.6f\n",caphis,cage[fage],0,' ',nsrv);}
        }
    }
}
void move(double n, int per, int area, int age, int fage, char *caphis)
{ int i=0,nxtage; double inarea,sum;
    if (debug) printf("mov:n=%f per=%d area=%d age=%d fage=%d caphis=%s psi=%f\n",
                      n,   per,   area,   age,   fage,   caphis,   mv[age][per][area][i]);
    nxtage=age+iage[per]; if (nxtage>=nages) nxtage=nages-1;
    for (i=0,sum=0; i<nareas-1; i++,sum+=inarea) {
        inarea=n*mv[age][per][area][i];
        cap(inarea,per+1,i,nxtage,fage,caphis);
    }
    if (n-sum>EPS) cap(n-sum,per+1,nareas-1,nxtage,fage,caphis);
}
void cap(double n, int per, int area, int age, int fage, char *caphis)
{ int i; double notcap, ncap, ndead; char *newcaphis;
    if (debug) printf("cap:n=%f per=%d caphis=%s age=%d fage=%d p=%f\n",n,per,caphis,age,fage,pa[age][per][area]);
               /* allocate memory for string */
    if ((newcaphis = (char *) malloc(MXPER+1)) == NULL) { printf("Not enough memory to allocate buffer\n"); exit(1); }
    if (age>nages-1) age=nages-1;
    strcpy(newcaphis,caphis); strcat(newcaphis,"0");
    notcap=n*(1.-pa[age][per][area]);
    srv(notcap,per,area,age,fage,newcaphis);
    ncap=n-notcap; if (fage<0) fage=age;
    newcaphis[strlen(newcaphis)-1]='A'+area;
    if (debug) printf("cap2:n=%f per=%d age=%d fage=%d caphis=%s\n",ncap,per,age,fage,newcaphis);
    srv(ncap,per,area,age,fage,newcaphis);
    free(newcaphis);
}
int main(int argc, char **argv)
{ char *caphis,tmpstr[256]; int i,j,k,kk,a; double sum,ba; FILE *f1;

    if (argc<2) { printf("usage genmsstl inputfile [debug]\n"); exit(1); }
    else  printf("genmsstl:\n");
    if (strcmp(argv[1],"help")==0)  {
        printf("\nINPUT FILE FORMAT:\n------------------\n");
        printf("nper nareas nages\n");
        printf("age-increments (1..nper-1)\n");
        printf("phi p (age=1,per=1,strata=A) mov(AB) mov(AC) ... lambda(AA) lambda(AB) ...\n");
        printf("phi p (age=1,per=1,strata=B) mov(BA) mov(BC) ... lambda(AA) lambda(AB) ...\n :\n");
        printf("phi p (age=1,per=2,strata=A) mov(AB) mov(AC) ... lambda(AA) lambda(AB) ...\n");
        printf("phi p (age=1,per=2,strata=B) mov(BA) mov(BC) ... lambda(AA) lambda(AB) ...\n :\n");
        printf("N1 N2 N3 ...N(nper)  (age=1,strata=A)\n");
        printf("N1 N2 N3 ...N(nper)  (age=1,strata=B)\n :\n");
        exit(1);
    }          
  if ((fp=fopen("genmsstl.out","w"))==NULL) { printf("error opening genmsstl.out\n"); exit(1); }
  if ((f1=fopen(argv[1],"r"))==NULL)  { printf("error opening inputfile(%s)\n",argv[1]); exit(1); }
  
  if (argc>2) debug=1;
  fgets(tmpstr,256,f1); sscanf(tmpstr,"%d %d %d",&nper,&nareas,&nages); determ=1;
  if (nper<0) {nper=-nper; determ=0;}
    printf("nper=%d nareas=%d nages=%d\n",nper,nareas,nages);
    if (nages==3) strcpy(cage,"jsa");
    if (nages>3) strcpy(cage,"jsabcdefghi");

  nper1=nper-1;

  if ((iage = (int *) malloc(sizeof(int)*(nper1))) == NULL)
	{ printf("error allocating memory for iage\n"); exit(1);  }

  printf("Age-increments:");
  for (i=0; i<nper1; i++)  { fscanf(f1,"%d",&iage[i]); printf(" %d",iage[i]); }
  printf("\n");

  if ((pa = (double ***) malloc(sizeof(double **)*(nages+1))) == NULL)
	{ printf("error allocating memory for pa\n"); exit(1); }
  for (i=0; i<nages+1; i++)
    { if ((pa[i] = (double **) malloc(sizeof(double *)*(nper+1))) == NULL)
    	  { printf("error allocating memory for pa\n"); exit(1); }
      for (j=0; j<nper+1; j++)
        if ((pa[i][j] = (double *) malloc(sizeof(double)*(nareas+1))) == NULL)
          { printf("error allocating memory for pa\n"); exit(1); }
    }

  if ((sa = (double ***) malloc(sizeof(double **)*(nages+1))) == NULL)
	{ printf("error allocating memory for sa\n"); exit(1);}
  for (i=0; i<nages+1; i++)
    { if ((sa[i] = (double **) malloc(sizeof(double *)*(nper+1))) == NULL)
    	  { printf("error allocating memory for sa\n"); exit(1); }
      for (j=0; j<nper+1; j++)
        if ((sa[i][j] = (double *) malloc(sizeof(double)*(nareas+1))) == NULL)
    	    { printf("error allocating memory for sa\n"); exit(1); }
    }

  if ((mv = (double ****) malloc(sizeof(double ***)*(nages+1))) == NULL)
	{ printf("error allocating memory for mv\n"); exit(1); }
  for (i=0; i<nages+1; i++)
    { if ((mv[i] = (double ***) malloc(sizeof(double **)*(nper+1))) == NULL)
    	 { printf("error allocating memory for mv\n"); exit(1); }
      for (j=0; j<nper+1; j++)
        { if ((mv[i][j] = (double **) malloc(sizeof(double *)*(nareas+1))) == NULL)
    	     { printf("error allocating memory for mv\n"); exit(1); }
          for (k=0; k<nareas+1; k++)
            { if ((mv[i][j][k] = (double *) malloc(sizeof(double)*(nareas+1))) == NULL)
                { printf("error allocating memory for mv\n"); exit(1); }
            }
        }
    }
  if ((lam = (double ****) malloc(sizeof(double ***)*(nages+1))) == NULL)
	{ printf("error allocating memory for lam\n"); exit(1); }
  for (i=0; i<nages+1; i++)
    { if ((lam[i] = (double ***) malloc(sizeof(double **)*(nper+1))) == NULL)
    	 { printf("error allocating memory for lam\n"); exit(1); }
      for (j=0; j<nper+1; j++)
        { if ((lam[i][j] = (double **) malloc(sizeof(double *)*(nareas+1))) == NULL)
    	     { printf("error allocating memory for lam\n"); exit(1); }
          for (k=0; k<nareas+1; k++)
            { if ((lam[i][j][k] = (double *) malloc(sizeof(double)*(nareas+1))) == NULL)
                { printf("error allocating memory for lam\n"); exit(1); }
            }
        }
    }
    for (a=0; a<nages; a++)
        for (i=0; i<nper; i++)
            for (j=0; j<nareas; j++) { 
                fscanf(f1,"%lf %lf",&sa[a][i][j],&pa[a][i][j]);
                if (sa[a][i][j]>1.0) sa[a][i][j]=1.0;
                if (pa[a][i][j]>1.0) pa[a][i][j]=1.0;
                for (k=0,sum=0; k<nareas-1; k++) { 
                    kk=k; if (k>=j) kk=k+1;
                    fscanf(f1,"%lf",&mv[a][i][j][kk]);
                    sum+=mv[a][i][j][kk];
                }
                mv[a][i][j][j]=1-sum;
                for (k=0;k<nareas; k++) fscanf(f1,"%lf",&lam[a][i][j][k]);
            }
    fprintf(f1,"\n");
    
    for (a=0; a<nages; a++)
        for (i=0; i<nper; i++) {
            for (j=0; j<nareas; j++) { 
                printf("%d %d %d: %f %f",a,i,j,sa[a][i][j],pa[a][i][j]);
                for (k=0,sum=0; k<nareas; k++) printf(" %f",mv[a][i][j][k]);
                for (k=0,sum=0; k<nareas; k++) printf(" %f",lam[a][i][j][k]);
                printf("\n");
            }
            printf("\n");
        }
    printf("\n"); 
    
    printf("srv,cap & movement rates:\n");
    printf("age time strata:      n      srv       p      move             recv.rate\n");
    for (a=0; a<nages; a++)
        for (j=0; j<nareas; j++)
            for (i=0; i<nper; i++) {
                fscanf(f1,"%lf",&ba);
                printf("%3d %4d %6d: %9.2f %8.4f %8.4f",a,i,j,ba,sa[a][i][j],pa[a][i][j]);
                for (k=0,sum=0; k<nareas; k++) printf(" %8.4f",mv[a][i][j][k]);
                for (k=0,sum=0; k<nareas; k++) printf(" %8.4f",lam[a][i][j][k]);
                printf("\n");
                if ((caphis=(char *) malloc(MXPER+1))==NULL) { printf("Not enough memory to allocate buffer\n"); exit(1); }
                caphis[0]='\0'; fper=i;
                if (ba>0) cap(ba,i,j,a,-1,caphis);
                free(caphis);
            }
    i=fclose(fp);
/************************* sample input file: **************
4 2
.8  .4  .7 
.85 .0  .35 
.7  .45 .65 
.75 .0  .60
.6  .55 .45 
.65 .0  .40
.0  .55 .00 
.0  .0  .00
1000 1000 1000 1000
1000 1000 1000 1000
*************************************************************/
}
