#include <math.h>
#define dabs(x) (x<0 ? -x : x)
double ndtr(double x) {
/* C  *DECK NDTR
C        SUBROUTINE NDTR
C
C        PURPOSE
C           COMPUTES Y = P(X) = PROBABILITY THAT THE RANDOM VARIABLE  U,
C           DISTRIBUTED NORMALLY(0,1), IS LESS THAN OR EQUAL TO X.
C           F(X), THE ORDINATE OF THE NORMAL DENSITY AT X, IS ALSO
C           COMPUTED.
C
C        USAGE
C           CALL NDTR(X,P)
C
C        DESCRIPTION OF PARAMETERS
C           X--INPUT SCALAR FOR WHICH P(X) IS COMPUTED.
C           P--OUTPUT PROBABILITY.
C
C        REMARKS
C           MAXIMUM ERROR IS 0.0000007.
C
C        SUBROUTINES AND SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           BASED ON APPROXIMATIONS IN C. HASTINGS, APPROXIMATIONS FOR
C           DIGITAL COMPUTERS, PRINCETON UNIV. PRESS, PRINCETON, N.J.,
C           1955.  SEE EQUATION 26.2.17, HANDBOOK OF MATHEMATICAL
C           FUNCTIONS, ABRAMOWITZ AND STEGUN, DOVER PUBLICATIONS, INC.,
C           NEW YORK.
C
   SUBROUTINE NDTR (X,P) */
    double p,t,d;
    t=1./(1.+.2316419*dabs(x));
    if (-x*x*0.5<-700) d=0.3989423*exp(-700.);
    else d=0.3989423*exp(-x*x*0.5);
    p=1.0-d*t*((((1.330274*t-1.821256)*t+1.781478)*t-0.3565638)*t+0.3193815);
    if (x<0.) p=1.-p;
    return(p);
}
